Package: FuseSOM
Title: A Correlation Based Multiview Self Organizing Maps Clustering
        For IMC Datasets
Version: 1.15.0
Authors@R: 
    person("Elijah", "Willie", , "ewil3501@uni.sydney.edu.au", role = c("aut", "cre"))
Description: A correlation-based multiview self-organizing map for the
        characterization of cell types in highly multiplexed in situ
        imaging cytometry assays (`FuseSOM`) is a tool for unsupervised
        clustering. `FuseSOM` is robust and achieves high accuracy by
        combining a `Self Organizing Map` architecture and a
        `Multiview` integration of correlation based metrics. This
        allows FuseSOM to cluster highly multiplexed in situ imaging
        cytometry assays.
License: GPL-2
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.2.2
Imports: psych, FCPS, analogue, coop, pheatmap, ggplotify, fastcluster,
        fpc, ggplot2, stringr, ggpubr, proxy, cluster, diptest,
        methods, SummarizedExperiment, stats, S4Vectors
LazyData: false
BuildResaveData: false
Depends: R (>= 4.2.0)
Suggests: knitr, BiocStyle, rmarkdown, SingleCellExperiment
VignetteBuilder: knitr
BugReports: https://github.com/ecool50/FuseSOM/issues
LinkingTo: Rcpp
biocViews: SingleCell, CellBasedAssays, Clustering, Spatial
Config/pak/sysreqs: cmake make libicu-dev libuv1-dev zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:59:22 UTC
RemoteUrl: https://github.com/bioc/FuseSOM
RemoteRef: HEAD
RemoteSha: e6e84914d8f60031513b6e205c7abb684b9d1aa3
NeedsCompilation: yes
Packaged: 2026-05-30 08:25:43 UTC; root
Author: Elijah Willie [aut, cre]
Maintainer: Elijah Willie <ewil3501@uni.sydney.edu.au>
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-30 08:29:18 UTC; unix

# Generated by roxygen2: do not edit by hand

export(clusterPrototypes)
export(computeGridSize)
export(estimateNumCluster)
export(generatePrototypes)
export(markerHeatmap)
export(normaliseData)
export(optiPlot)
export(runFuseSOM)
import(ggplot2)
import(ggpubr)
import(stringr)
importFrom(FCPS,HierarchicalClustering)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,metadata)
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,colData)
importFrom(analogue,fuse)
importFrom(coop,tcosine)
importFrom(ggplotify,as.ggplot)
importFrom(methods,is)
importFrom(pheatmap,pheatmap)
importFrom(proxy,dist)
importFrom(psych,cor2dist)
importFrom(stats,aggregate)
importFrom(stats,cor)
importFrom(stats,cov.wt)
importFrom(stats,kmeans)
importFrom(stats,prcomp)
importFrom(stats,princomp)
importFrom(stats,quantile)
importFrom(stats,sd)
useDynLib(FuseSOM)

FuseSOM
======================================================

<img src=https://raw.githubusercontent.com/ecool50/FuseSOM/main/inst/FuseSOM.png align="right" height="200">


A correlation based `Multiview` Self Organizing Map for the characterization of 
cell types in `highly multiplexed in situ imaging cytometry assays`.

Overview
--------

**FuseSOM** provides a pipeline for the clustering of `highly multiplexed in situ imaging cytometry assays`.
This pipeline uses the `Self Organizing Map` architecture coupled with `Multiview` hierarchical clustering.
We also provide functions for normalisation and estimation of the number of clusters.

Installation
--------

Install the Bioconductor release version.

```r
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("FuseSOM")
```
Otherwise, if you would like the most up-to-date features, install the development version.

```r
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

# The following initializes usage of Bioc devel
BiocManager::install(version='devel')

BiocManager::install("FuseSOM")

# Or install the development version from GitHub:
# install.packages("devtools")
devtools::install_github("https://github.com/ecool50/FuseSOM")
library(FuseSOM)
```

### Submitting an issue or feature request

`FuseSOM` is still under active development. We would greatly appreciate any and 
all feedback related to the package.

* R package related issues should be raised [here](https://github.com/ecool50/FuseSOM/issues).
* For general questions and feedback, please contact us directly via [ewil3501@uni.sydney.edu.au](mailto:ewil3501@uni.sydney.edu.au).


## Author

* **Elijah Willie**
* **Ellis Patrick**  - [@TheEllisPatrick](https://twitter.com/TheEllisPatrick)

## ----knitr-options, echo=FALSE, message=FALSE, warning=FALSE------------------
library(knitr)
opts_chunk$set(fig.align = 'center', fig.width = 6, fig.height = 5, dev = 'png')

## ----eval = FALSE-------------------------------------------------------------
# if (!require("BiocManager"))
#     install.packages("BiocManager")
# BiocManager::install("FuseSOM")

## ----message=FALSE, warning=FALSE---------------------------------------------
# load FuseSOM
library(FuseSOM)


## -----------------------------------------------------------------------------
# load in the data
data("risom_dat")

# define the markers of interest
risomMarkers <- c('CD45','SMA','CK7','CK5','VIM','CD31','PanKRT','ECAD',
                   'Tryptase','MPO','CD20','CD3','CD8','CD4','CD14','CD68','FAP',
                   'CD36','CD11c','HLADRDPDQ','P63','CD44')

# we will be using the manual_gating_phenotype as the true cell type to gauge 
# performance
names(risom_dat)[names(risom_dat) == 'manual_gating_phenotype'] <- 'CellType'


## -----------------------------------------------------------------------------
risomRes <- runFuseSOM(data = risom_dat, markers = risomMarkers, 
                        numClusters = 23)

## -----------------------------------------------------------------------------
# get the distribution of the clusters
table(risomRes$clusters)/sum(table(risomRes$clusters))


## -----------------------------------------------------------------------------
risomHeat <- FuseSOM::markerHeatmap(data = risom_dat, markers = risomMarkers,
                            clusters = risomRes$clusters, clusterMarkers = TRUE)

## -----------------------------------------------------------------------------
# lets estimate the number of clusters using all the methods
# original clustering has 23 clusters so we will set kseq from 2:25
# we pass it the som model generated in the previous step
risomKest <- estimateNumCluster(data = risomRes$model, kSeq = 2:25, 
                                  method = c("Discriminant", "Distance"))


## -----------------------------------------------------------------------------
# what is the best number of clusters determined by the discriminant method?
# optimal number of clusters according to the discriminant method is 7
risomKest$Discriminant 

# we can plot the results using the optiplot function
pSlope <- optiPlot(risomKest, method = 'slope')
pSlope
pJump <- optiPlot(risomKest, method = 'jump')
pJump
pWcd <- optiPlot(risomKest, method = 'wcd')
pWcd
pGap <- optiPlot(risomKest, method = 'gap')
pGap
pSil <- optiPlot(risomKest, method = 'silhouette')
pSil


## ----message=FALSE, warning=FALSE---------------------------------------------
library(SingleCellExperiment)

# create a singelcellexperiment object
colDat <- risom_dat[, setdiff(colnames(risom_dat), risomMarkers)]
sce <- SingleCellExperiment(assays = list(counts = t(risom_dat)),
                                 colData = colDat)

sce

## -----------------------------------------------------------------------------
risomRessce <- runFuseSOM(sce, markers = risomMarkers, assay = 'counts', 
                      numClusters = 23, verbose = FALSE)

colnames(colData(risomRessce))
names(metadata(risomRessce))

## -----------------------------------------------------------------------------
data <- risom_dat[, risomMarkers] # get the original data used
clusters <- colData(risomRessce)$clusters # extract the clusters from the sce object
# generate the heatmap
risomHeatsce <- markerHeatmap(data = risom_dat, markers = risomMarkers,
                            clusters = clusters, clusterMarkers = TRUE)

## -----------------------------------------------------------------------------
# lets estimate the number of clusters using all the methods
# original clustering has 23 clusters so we will set kseq from 2:25
# now we pass it a singlecellexperiment object instead of the som model as before
# this will return a singelcellexperiment object where the metatdata contains the
# cluster estimation information
risomRessce <- estimateNumCluster(data = risomRessce, kSeq = 2:25, 
                                  method = c("Discriminant", "Distance"))

names(metadata(risomRessce))

## -----------------------------------------------------------------------------
# what is the best number of clusters determined by the discriminant method?
# optimal number of clusters according to the discriminant method is 8
metadata(risomRessce)$clusterEstimation$Discriminant 

# we can plot the results using the optiplot function
pSlope <- optiPlot(risomRessce, method = 'slope')
pSlope
pJump <- optiPlot(risomRessce, method = 'jump')
pJump
pWcd <- optiPlot(risomRessce, method = 'wcd')
pWcd
pGap <- optiPlot(risomRessce, method = 'gap')
pGap
pSil <- optiPlot(risomRessce, method = 'silhouette')
pSil


## -----------------------------------------------------------------------------
sessionInfo()

# A function to generate the grid for the self organizing map
# function was obtained from https://rdrr.io/rforge/yasomi/ with some major modifications
#' @importFrom proxy dist
somGrid <- function(xDim, yDim, topo = c("rectangular", "hexagonal"), with.dist = TRUE) {
  topo <- match.arg(topo)
  if (xDim == 1) {
    tmp <- xDim
    xDim <- yDim
    yDim <- tmp
  }
  x <- seq(from = 1, by = 1, length.out = xDim)
  if (topo == "hexagonal" && yDim > 1) {
    y <- rev(seq(from = 1, by = sqrt(3) / 2, length.out = yDim))
  } else {
    y <- rev(seq(from = 1, by = 1, length.out = yDim))
  }
  pts <- as.matrix(expand.grid(x = x, y = y))
  if (topo == "hexagonal" && yDim > 1) {
    pts[, 1] <- pts[, 1] + rep(c(0, 0.5), each = xDim, length.out = nrow(pts))
  }
  if (topo == "rectangular") {
    diam <- sqrt(xDim^2 + yDim^2)
  } else {
    diam <- sqrt(sum((pts[1, ] - pts[nrow(pts), ])^2))
  }
  res <- list(
    pts = pts, xDim = xDim, yDim = yDim, topo = topo,
    size = xDim * yDim, diam = diam
  )
  if (with.dist) {
    res$dist <- as.matrix(dist(pts, method = "Euclidean"), diag = 0)
  }
  class(res) <- "somgrid"
  res
}

# A function to estimate the number of clusters using the distance method
# see https://arxiv.org/abs/1608.07494
# function was obtained from https://github.com/cran/cstab with some
# minor modifications

.cDistance <- function(data, # n x p data matrix
                       kSeq, # sequence of ks to be checked
                       linkage = "average",
                       gapIter = 10) # n of simulated datasets in gap statistic
{

  # ----- INPUT TESTS

  # On Data
  if (sum(is.na(data)) > 0) stop("No missing values permitted!")

  # On k-sequence
  if (1 %in% kSeq) {
    stop("Please select a k sequence starting with 2: {2,3,...K}!")
  }


  # ----- HELPERS

  n <- nrow(data)
  dims <- ncol(data)
  if (!1 %in% kSeq) kSeq <- c(1, kSeq)


  # ----- EVALUATE REAL DATA

  WCD <- Sil <- MSE <- numeric()
  for (k in kSeq) {
    obj <- .getMeasures(data = data, k = k)
    WCD[k] <- obj$WCD
    Sil[k] <- obj$Sil
    MSE[k] <- obj$MSE
  }


  # ----- EVALUATE SYNTHETIC DATA (Gap-statistic)

  WCDRuns <- matrix(NA, nrow = gapIter, ncol = length(kSeq))
  for (i in seq_len(gapIter)) {
    dataSyn <- .uniformData(data)
    WCDs <- numeric()
    for (j in seq_along(kSeq)) {
      k <- kSeq[j]
      obj <- .getMeasures(data = dataSyn, k = k, measures = c("wcd"))
      WCDs[j] <- obj$WCD
    }
    WCDRuns[i, ] <- WCDs
  }
  WCDSyn <- colMeans(WCDRuns)

  # ----- COMPUTE MEASURES

  # Gap Statistic
  WCDDatLog <- log(WCD)
  WCDSynLog <- log(WCDSyn)
  WCDDatLog <- WCDDatLog - WCDDatLog[1]
  WCDSynLog <- WCDSynLog - WCDSynLog[1]
  gap <- WCDSynLog - WCDDatLog
  koptGap <- kSeq[gap == max(gap)]


  # Slope Statistic
  p <- 1
  slope <- -(Sil[-1] - Sil[-length(Sil)]) * Sil[-1]^p
  koptSlope <- kSeq[slope == max(slope)]

  ## Jump Statistic
  MSETr <- MSE^(-dims / 2)
  jump <- (MSETr - c(0, MSETr[-length(MSETr)])) # [-1]
  koptJump <- kSeq[jump == max(jump)]

  outlist <- list(
    "kGap" = koptGap,
    "kSlope" = koptSlope,
    "kJump" = koptJump,
    "WCD" = WCD,
    "WCDSyn" = WCDSyn,
    "Gaps" = gap,
    "Silhouettes" = Sil,
    "Slopes" = slope,
    "Jumps" = jump
  )

  return(outlist)
} # EoF

#' FuseSOM
#' 
#' FuseSOM provides a pipeline for the clustering of highly
#' multiplexed in situ imaging cytometry assays. This pipeline uses
#' the Self Organizing Map architecture coupled with Multiview hierarchical
#' clustering. We also provide functions for normalisation and estimation
#' of the number of clusters.
#'
#' The FuseSOM package provides three categories of important functions:
#' foo, bar and baz.
#'
#'
#' @docType package
#' @name FuseSOM
#' @useDynLib FuseSOM
NULL
#> NULL

#' these functions were obtained from https://rdrr.io/rforge/yasomi/
#' with some major modifications
#'
#' @param data The data to which the SOM will be fitted, a matrix or data frame
#'             of observations (which should be scaled)
#' @param somGrid A `somgrid` object
#' @param weights Optional weights for the data points
#' @param with.princomp Switch specifying whether the princomp should be used
#'                      instead of the prcomp for computing the principal
#'                      components when no weights are given (see details)
#' @param ... not used
#' @return A list containing: `prototype`, a matrix containing appropriate
#'         initial prototypes, and `data.pca` the results of the PCA conducted
#'         on the data
#'
#' @importFrom stats princomp prcomp cov.wt
somInitPca.default <- function(data,
                               somGrid,
                               weights,
                               with.princomp = FALSE,
                               ...) {
  ### FIXME: data weights support
  ## we don't scale the data
  if (missing(weights) || is.null(weights)) {
    if (with.princomp) {
      dataPca <- princomp(covmat = cov.wt(data))
    } else {
      dataPca <- prcomp(data)
    }
  } else {
    dataPca <- princomp(covmat = cov.wt(data, weights))
  }
  ## the more detailled axis is assigned to the first eigenvector
  if (somGrid$xDim >= somGrid$yDim) {
    xEv <- 1
    yEv <- 2
  } else {
    xEv <- 2
    yEv <- 1
  }
  if (somGrid$topo == "hexagonal") {
    xSpan <- somGrid$xDim - 1
    if (somGrid$yDim > 1) {
      xSpan <- xSpan + 0.5
    }
    x <- seq(
      from = -2 * dataPca$sdev[xEv],
      by = 4 * dataPca$sdev[xEv] / xSpan,
      length.out = somGrid$xDim
    )
  } else {
    x <- seq(
      from = -2 * dataPca$sdev[xEv],
      to = 2 * dataPca$sdev[xEv],
      length.out = somGrid$xDim
    )
  }
  y <- seq(
    from = 2 * dataPca$sdev[yEv],
    to = -2 * dataPca$sdev[yEv],
    length.out = somGrid$yDim
  )
  base <- as.matrix(expand.grid(x = x, y = y))
  ## correction for hexagonal grids
  if (somGrid$topo == "hexagonal") {
    base[, 1] <- base[, 1] + rep(
      c(0, 2 * dataPca$sdev[xEv] / xSpan),
      each = somGrid$xDim,
      length.out = nrow(base)
    )
  }
  ## map back the grid to the original space
  if (inherits(dataPca, "prcomp")) {
    mapped <- tcrossprod(base, dataPca$rotation[, c(xEv, yEv)])
  } else {
    mapped <- tcrossprod(base, dataPca$loadings[, c(xEv, yEv)])
  }
  ## decentering
  prototypes <- sweep(mapped, 2, dataPca$center, "+")
  list(prototypes = prototypes, dataPca = dataPca)
}

bmu <- function(prototypes, data, weights) {
  if (ncol(prototypes) != ncol(data)) {
    stop("'prototypes' and 'data' have different dimensions")
  }
  if (missing(weights)) {
    weights <- rep(1, nrow(data))
  } else if (length(weights) != nrow(data)) {
    stop("'weights' and 'data' have different dimensions")
  }
  result <- .C("bmu",
    as.double(prototypes),
    as.integer(nrow(prototypes)),
    as.double(data),
    as.integer(nrow(data)),
    as.integer(ncol(prototypes)),
    as.double(weights),
    clusters = integer(nrow(data)),
    error = as.double(0),
    PACKAGE = "FuseSOM"
  )
  list(clusters = result$clusters + 1, error = result$error)
}

secondBmu <- function(prototypes, data) {
  if (ncol(prototypes) != ncol(data)) {
    stop("'prototypes' and 'data' have different dimensions")
  }
  matrix(.C("second_bmu",
    as.double(prototypes),
    as.integer(nrow(prototypes)),
    as.double(data),
    as.integer(nrow(data)),
    as.integer(ncol(prototypes)),
    clusters = integer(2 * nrow(data)),
    PACKAGE = "FuseSOM"
  )$clusters, ncol = 2)
}

batchSom.default <- function(data, somGrid,
                             init = c("pca", "random"), prototypes,
                             weights,
                             mode = c("continuous", "stepwise"),
                             minRadius, maxRadius, steps,
                             decrease = c("power", "linear"), maxIter,
                             kernel = c("gaussian", "linear"), normalised,
                             assignment = c("single", "heskes"),
                             cut = 1e-07,
                             verbose = FALSE, keepdata = TRUE, ...) {
  theCall <- match.call()
  if (verbose) {
    message(theCall)
  }
  theCall[[1]] <- batchSomControl
  control <- eval(theCall, envir = parent.frame())
  control$assignmentInt <- switch(control$assignment,
    "single" = 0,
    "heskes" = 1
  )
  control$kernelInt <- switch(control$kernel,
    "gaussian" = 0,
    "linear" = 1
  )
  if (!missing(weights)) {
    if (length(weights) != nrow(data)) {
      stop("'weights' and 'data' have different dimensions")
    }
  } else {
    ## keep weights to NULL for now to avoid princomp initialization
    weights <- NULL
  }
  if (missing(prototypes)) {
    ## initialisation based on the value of init
    init <- match.arg(init)
    args <- list(...)
    params <- c(list(data = data, somGrid = somGrid, weights = weights), args)
    prototypes <- switch(init,
      "pca" = do.call("somInitPca", params)$prototypes,
      "random" = do.call("somInitRandom", params)
    )
  } else {
    if (ncol(prototypes) != ncol(data)) {
      stop("'prototypes' and 'data' have different dimensions")
    }
    if (nrow(prototypes) != somGrid$size) {
      stop("'prototypes' and 'somgrid' are not compatible")
    }
  }
  if (is.null(weights)) {
    weights <- rep(1, nrow(data))
  }
  ## distances?
  if (is.null(somGrid$dist)) {
    somGrid$dist <- as.matrix(dist(somGrid$pts, method = "Euclidean"), diag = 0)
  }
  pre <- switch(control$mode,
    "stepwise" = batchSomLowLevel(
      somGrid, data, prototypes, weights,
      control, verbose
    ),
    "continuous" = batchSomLowLevelContinuous(
      somGrid, data,
      prototypes, weights, control, verbose
    )
  )
  pre$control <- control
  if (keepdata) {
    pre$data <- data
    pre$weights <- weights
  }
  pre
}

batchSomLowLevel <- function(somGrid, data, prototypes,
                             weights, control, verbose) {
  result <- .C("batch_som_optim",
    proto = as.double(prototypes),
    as.integer(somGrid$size),
    as.double(data),
    as.integer(nrow(data)),
    as.integer(ncol(data)),
    as.double(weights),
    as.integer(control$assignmentInt),
    as.double(somGrid$dist),
    as.double(control$radii),
    as.integer(length(control$radii)),
    as.integer(control$maxIter),
    as.integer(control$kernelInt),
    as.integer(control$normalised),
    as.double(control$cut),
    as.integer(verbose),
    clusters = integer(nrow(data)),
    errors = as.double(rep(-1, 1 + length(control$radii) * control$maxIter)),
    PACKAGE = "FuseSOM"
  )
  prototypes <- matrix(result$proto,
    ncol = ncol(prototypes),
    dimnames = list(NULL, dimnames(data)[[2]])
  )
  res <- list(
    somGrid = somGrid,
    prototypes = prototypes,
    classif = result$cluster + 1,
    errors = result$errors[result$errors >= 0]
  )
  class(res) <- c("somnum", "som")
  res
}

batchSomLowLevelContinuous <- function(somGrid, data, prototypes, weights,
                                       control, verbose) {
  result <- .C("batch_som_optim_continuous",
    proto = as.double(prototypes),
    as.integer(somGrid$size),
    as.double(data),
    as.integer(nrow(data)),
    as.integer(ncol(data)),
    as.double(weights),
    as.integer(control$assignmentInt),
    as.double(somGrid$dist),
    as.double(control$radii),
    as.integer(length(control$radii)),
    as.integer(control$kernelInt),
    as.integer(control$normalised),
    as.double(control$cut),
    as.integer(verbose),
    clusters = integer(nrow(data)),
    errors = as.double(rep(-1, 1 + length(control$radii))),
    PACKAGE = "FuseSOM"
  )
  prototypes <- matrix(result$proto,
    ncol = ncol(prototypes),
    dimnames = list(NULL, dimnames(data)[[2]])
  )
  res <- list(
    somGrid = somGrid,
    prototypes = prototypes,
    classif = result$cluster + 1,
    errors = result$errors[result$errors >= 0]
  )
  class(res) <- c("somnum", "som")
  res
}

#' Discriminant cluster estimator
#' 
#' Function to estimate the number of clusters using discriminant analysis
#' parts of this function is based on the sigclust2 package by Patrick Kimes
#' see https://github.com/pkimes/sigclust2
#'
#' @param distMat A distance matrix
#' @param minClusterSize The minimum cluster size
#' @param alpha a value between 0 and 1 specifying the desired level of cutoff
#' @return Optimal number of clusters
#'
#' @importFrom stats kmeans
.runDiscriminant <- function(distMat, minClusterSize, alpha = 0.001) {

  # do some house keeping
  n <- nrow(distMat)
  p <- ncol(distMat)
  ndType <- rep("", n - 1)
  pEmp <- rep(0, n - 1)

  numClusters <- 0

  # generate the initial tree
  initTree <- fastcluster::hclust(
    dist(distMat, method = "maximum"),
    method = "average"
  )

  # process the resulting dendrogram
  hcDat <- initTree
  idxHC <- .idxHc(initTree, n)
  cutoff <- .fwerCutoffmatrix(idxHC, alpha)
  pdMap <- .pdMap(initTree, n)
  ndType <- rep("", n - 1)

  # run significance testing on each node
  for (k in seq_len(n - 1)) {
    ## indices for subtree
    idxVals <- idxHC[k, ]
    idxSub <- unlist(idxHC[k, ])
    nSub <- length(idxSub)

    ## only calc p-values for branches w/ more than n_mini
    if (nSub < minClusterSize) {
      ndType[k] <- "n_small"
      next
    }

    if ((alpha < 1) && (k > 1) && (ndType[pdMap[k]] != "sig")) {
      ndType[k] <- "no_test"
      pEmp[k] <- 1
      next
    }

    # Generate initial assingments
    t <- c(idxVals[[1]], idxVals[[2]])
    xComb <- distMat[t, t]
    assignments <- kmeans(xComb, 2)$cluster

    # compute the discriminant projections
    xNew <- fpc::discrcoord(x = xComb, clvecd = assignments)$proj[, 1]
    resPval <- diptest::dip.test(xNew)$p.value

    # update results
    if (alpha < 1) {
      if (resPval < alpha) {
        ndType[k] <- "sig"
        numClusters <- numClusters + 1
      } else {
        ndType[k] <- "not_sig"
      }
      pEmp[k] <- resPval
    }
  }
  return(numClusters)
}

# identify parent node of each node in dendrogram
# parts of this function is based on the sigclust2 package by Patrick Kimes
# see https://github.com/pkimes/sigclust2
.pdMap <- function(hc, n) {
  ## determine parent branch node for all children nodes along dendrogram
  pdPairs <- rbind(
    cbind(hc$merge[, 1], seq_len(n - 1)),
    cbind(hc$merge[, 2], seq_len(n - 1))
  )
  pdMap <- data.frame(pdPairs[pdPairs[, 1] > 0, ])
  names(pdMap) <- c("dtr", "prt")
  pdMap <- pdMap$prt[order(pdMap$dtr)] # the parent of each daughter
  pdMap <- c(pdMap, n) # add final node without a parent

  ## flip index, hclust and shc use reversed ordering
  n - rev(pdMap)
}


## determine obs indices at each node of the dendrogram
# parts of this function is based on the sigclust2 package by Patrick Kimes
# see https://github.com/pkimes/sigclust2
.idxHc <- function(hc, n) {
  ## list array of cluster indices at each of the n-1 merges
  idxHC <- array(list(), c(2 * n - 1, 2))
  idxHC[seq_len(n), 1] <- as.list(n:1)
  idxHC[(n + 1):(2 * n - 1), ] <- hc$merge + n + (hc$merge < 0)

  ## complete idxHC
  for (k in seq_len(n - 1)) {
    idxHC[[n + k, 1]] <- unlist(idxHC[idxHC[[n + k, 1]], ])
    idxHC[[n + k, 2]] <- unlist(idxHC[idxHC[[n + k, 2]], ])
  }

  ## flip index, hclust and shc use revered ordering
  idxHC[(2 * n - 1):(n + 1), ]
}

# parts of this function is based on the sigclust2 package by Patrick Kimes
# see https://github.com/pkimes/sigclust2
.fwerCutoffmatrix <- function(obj, alpha, ...) {
  alpha / (nrow(obj) + 1) *
    apply(obj, 1, function(x) {
      length(unlist(x))
    })
}

#' IMC Breast Cancer Data
#' Data from A spatial atlas of breast cancer progression using MIBI-TOF
#' and tissue transcriptomics
#' @usage data(risom_dat)
#'
#' @format An object of class `"data.frame"`.
#' @aliases
#' diabetesData
#'
#' @keywords datasets
#'
#' @references T. Risom, et al.
#' Transition to invasive breast cancer is associated with progressive changes
#' in the structure and composition of tumor stroma Cell, 185 (2022), pp. 299-310
#' ([ScienceDirect](https://www.sciencedirect.com/science/article/pii/S0092867421014860?via%3Dihub))
#'
#' @source Mendeley Data, <https://data.mendeley.com/datasets/d87vg86zd8/3>
#'
#'
"risom_dat"

somInitPca <- function(data, somgrid, ...) {
  UseMethod("somInitPca")
}

batchSom <- function(data, somGrid, init = c("pca"), prototypes, weights,
                     mode = c("continuous", "stepwise"),
                     minRadius, maxRadius, steps,
                     decrease = c("power", "linear"), maxIter,
                     kernel = c("gaussian", "linear"), normalised,
                     assignment = c("single", "heskes"),
                     cut = 1e-07,
                     verbose = FALSE, keepdata = TRUE, ...) {
  UseMethod("batchSom")
}

## in annealing.R

batchSomControl <- function(data, somGrid,
                            mode = c("continuous", "stepwise"),
                            minRadius, maxRadius, steps,
                            decrease = c("power", "linear"), maxIter,
                            kernel = c("gaussian", "linear"),
                            normalised,
                            assignment = c("single", "heskes"),
                            cut = 1e-07, ...) {
  UseMethod("batchSomControl")
}

#' Function to do percentile normalizaton
#'
#' @param x Matrix to percentile normilse.
#' @return percentile normalized version of `x`
#'
#' @importFrom stats quantile
.percentileNorm <- function(x) {
  x <- as.matrix(x)
  percentiles <- quantile(x, probs = c(0.01, 0.99))
  q1 <- percentiles[[1]]
  q99 <- percentiles[[2]]

  x <- (x - q1) / (q99)
  return(x)
}

#' Function to do min max normalization
#'
#' @importFrom stats quantile
#'
#' @param x Matrix to min max nomalize.
#' @return Max normalized version of `x`
.minmaxNorm <- function(x) {
  x <- as.matrix(x)
  percentiles <- quantile(x, probs = c(0.01, 0.99))
  minVal <- percentiles[[1]]
  maxVal <- percentiles[[2]]
  x <- (x - minVal) / (maxVal - minVal)
  return(x)
}

#' Function to do arsinh normalization
#'
#' @param x A numeric or complex vector
#' @param cofactor Cofactor of the vector. Default is 5.
#' @return Arsinh normalized vector.
.arsinhNnorm <- function(x, cofactor = 5) {
  x <- asinh(x / cofactor)
  return(x)
}

#' Creates uniformly distributed data of same dimensionality as input data
#' this function was obtained from the Stab package
#'
#' @param data A data matrix.
#' @return Uniform random noise with `dim(data)`
.uniformData <- function(data) {

  # get dimensions of data
  n <- nrow(data)
  unifDims <- t(apply(data, 2, range))

  # sample data and combine to data matrix
  dataSynt <- matrix(NA, nrow = nrow(data), ncol = ncol(data))
  for (i in seq_len(ncol(data))) {
    dataSynt[, i] <- stats::runif(n, unifDims[i, 1], unifDims[i, 2])
  }
  return(dataSynt)
}

#' A function to compute the elbow point given a set of points
#'
#' @param vals Values to compute the elbow point of.
#' @return A integer indicating the elbow point of `vals`.
.computeElbow <- function(vals) {
  diffs <- diff(vals)
  diffs <- diffs[-1]
  optKb <- which.max(abs(diffs)) + 1
  return(optKb)
}

# A function to generate the radii values for the Gaussian kernel function was
# obtained from https://rdrr.io/rforge/yasomi/ with some major modifications

radiusExp <- function(min, max, steps) {
  max * (min / max)^(seq(0, 1, length.out = steps))
}

radiusLin <- function(min, max, steps) {
  seq(max, min, length.out = steps)
}

batchSomControl.default <- function(data, somGrid,
                                    mode = c("continuous", "stepwise"),
                                    minRadius, maxRadius, steps,
                                    decrease = c("power", "linear"), maxIter,
                                    kernel = c("gaussian", "linear"),
                                    normalised,
                                    assignment = c("single", "heskes"),
                                    cut = 1e-07, ...) {
  mode <- match.arg(mode, c("continuous", "stepwise"))
  decrease <- match.arg(decrease, c("power", "linear"))
  kernel <- match.arg(kernel, c("gaussian", "linear"))
  assignment <- match.arg(assignment, c("single", "heskes"))
  if (missing(maxRadius)) {
    maxRadius <- 2 / 3 * somGrid$diam + 1
  }
  if (missing(minRadius)) {
    minRadius <- switch(kernel,
      "gaussian" = 0.5,
      "linear" = 1
    )
  }
  if (maxRadius <= minRadius) {
    stop("maxRadius must be larger than minRadius")
  }
  if (minRadius <= 0) {
    stop("minRadius must be positive")
  }
  if (missing(steps)) {
    steps <- switch(mode,
      "stepwise" = max(2, ceiling(2 * maxRadius)),
      "continuous" = max(20, ceiling(5 * maxRadius))
    )
  }
  if (missing(maxIter)) {
    maxIter <- switch(mode,
      "stepwise" = 75,
      "continuous" = 1
    )
  }
  if (missing(normalised)) {
    normalised <- assignment == "heskes"
  }
  radii <- switch(decrease,
    "linear" = radiusLin(minRadius, maxRadius, steps),
    "power" = radiusExp(minRadius, maxRadius, steps)
  )
  list(
    mode = mode, radii = radii, maxIter = maxIter, kernel = kernel,
    normalised = normalised, assignment = assignment,
    cut = cut
  )
}

# a function to compute the various measures when estimating the number of clusters
# see https://arxiv.org/abs/1608.07494
# function was obtained from https://github.com/cran/cstab with some minor modifications
.getMeasures <- function(data,
                         k,
                         linkage = "average",
                         measures = c("wcd", "sil", "mse")) {
  pear <- stats::cor(t(data), method = "pearson")
  cosi <- coop::tcosine(data)
  spear <- stats::cor(t(data), method = "spearman")

  dMat <- as.matrix(fuse(cor2dist(pear), cor2dist(cosi), cor2dist(spear)))

  hcObj <- fastcluster::hclust(stats::as.dist(dMat), method = linkage)
  cl <- stats::cutree(hcObj, k)

  # ------ WITHIN CLUSTER DISSIMILARITY

  # calc within cluster dissimilarity
  WCD <- NULL
  if ("wcd" %in% measures) {
    normDiss <- c()
    for (i in seq_len(k)) {
      normDiss[i] <- sum(dMat[cl == i, cl == i]) / (2 * sum(cl == i))
    }
    WCD <- sum(normDiss)
  }


  # ------ SILHOUETTE

  Sil <- NULL
  if ("sil" %in% measures) {
    if (k > 1) Sil <- mean(cluster::silhouette(cl, stats::as.dist(dMat))) else Sil <- 0
  }

  # TODO: Touch base with Elijah to ensure I haven't proken anything
  # by changing this line.
  # if('sil' %in% measures) if(k > 1) Sil <- mean(cluster::silhouette(cl, stats::as.dist(dMat))[,3]) else Sil <- 0

  # ------ CLUSTER CENTERS

  # get centers
  centers <- NULL
  if ("mse" %in% measures || "centers" %in% measures) {
    if (k == 1) centers <- colMeans(data)
    if (k > 1) {
      dataOr <- as.data.frame(data)
      dataOr$cl <- cl
      dataOr <- dataOr[order(dataOr$cl), ]
      centers <- matrix(NA, k, ncol(data))
      for (kk in seq_len(k)) {
        centers[kk, ] <- colMeans(dataOr[dataOr$cl == kk, -(ncol(data) + 1)])
      }
    }
  }
  # ------ MSE FOR JUMP

  MSE <- NULL
  if ("mse" %in% measures) {
    if (k == 1) {
      SE <- apply(data, 1, function(inst) {
        diffs <- inst - centers
        return(t(diffs) %*% diffs)
      })
    }
    if (k > 1) {
      SE <- numeric()
      for (i in seq_len(nrow(data))) {
        diffs <- data[i, ] - centers[cl[i], ]
        se <- t(diffs) %*% diffs
        SE[i] <- se
      }
    }
    MSE <- sum(SE) / nrow(data)
  }


  # ------ RETURN

  outlist <- list("WCD" = WCD, "Sil" = Sil, "MSE" = MSE, "Centers" = centers)
  return(outlist)
}

#' Estimate the optimal grid size
#'
#' The function finds the eigenvalues of the sample covariance matrix.
#' It will then return the number of significant eigenvalues according to
#' the Tracy-Widom test.
#' The function is based on the estKW function from the SC3 package
#' @param dataset The optimal grid size.
#' @return the optimal grid size.
#'
#' @examples
#' data("risom_dat")
#' risomMarkers <- c(
#'   "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
#' )
#' computeGridSize(risom_dat[, risomMarkers])
#'
#' @author
#'   Elijah WIllie <ewil3501@uni.sydney.edu.au>
#'
#' @export
#'
computeGridSize <- function(dataset) {
  p <- ncol(dataset)
  n <- nrow(dataset)

  # compute Tracy-Widom bound
  x <- scale(dataset)
  muTW <- (sqrt(n - 1) + sqrt(p))^2
  sigmaTW <- (sqrt(n - 1) + sqrt(p)) * (1 / sqrt(n - 1) + 1 / sqrt(p))^(1 / 3)
  sigmaHatNaive <- t(x) %*% x # x left-multiplied by its transpose
  # 3.2730 is the p=0.001 percentile point for the Tracy-Widom distribution
  bd <- 3.273 * sigmaTW + muTW

  # compute eigenvalues and return the amount which falls above the bound
  evals <- eigen(sigmaHatNaive, symmetric = TRUE, only.values = TRUE)$values
  k <- 0
  for (i in seq_along(evals)) {
    if (evals[i] > bd) {
      k <- k + 1
    }
  }
  return(k + 2)
}

#' Normalise Marker Intensities
#' 
#' The matrix of intensities is normalised based on one of four different method
#' These methods include Percentile, zscore, arsinh and minmax
#'
#' @param data the raw intensity scores.
#' @param markers the markers of interest.
#' @param  method the normalizaton method
#' @param cofactor the cofactor for arsinh normalisation
#' @return normalised matrix.
#'
#' @examples
#' data("risom_dat")
#' risomMarkers <- c(
#'   "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
#' )
#' normaliseData(risom_dat[, risomMarkers])
#'
#' @author
#'   Elijah WIllie <ewil3501@uni.sydney.edu.au>
#' @export
normaliseData <- function(data, markers, method = "none", cofactor = 5) {
  if (!(method %in% c("none", "percentile", "zscore", "arsinh", "minmax"))) {
    stop("Please provide a valid normalization method")
  }

  data <- as.matrix(data[, markers])

  if (method == "none") {
    return(data)
  } else if (method == "percentile") {
    return(.percentileNorm(data))
  } else if (method == "minmax") {
    return(.minmaxNorm(data))
  } else if (method == "arsinh") {
    return(.arsinhNnorm(data, cofactor))
  }
  return(scale(data))
}


#' Normalize Marker Intensities
#' 
#' The matrix of intensities is normalised based on one of four different method
#' These methods include Percentile, zscore, arsinh and minmax
#'
#' @param data the raw intensity scores.
#' @param markers the markers of interest.
#' @param method the normalizaton method
#' @param cofactor the cofactor for arsinh normalization
#' @return normalised matrix.
#'
#' @examples
#' data("risom_dat")
#' risomMarkers <- c(
#'   "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
#' )
#' normaliseData(risom_dat[, risomMarkers])
#'
#' @author
#'   Elijah WIllie <ewil3501@uni.sydney.edu.au>
normalizeData <- function(data, markers, method = "none", cofactor = 5) {
  if (!(method %in% c("none", "percentile", "zscore", "arsinh", "minmax"))) {
    stop("Please provide a valid normalization method")
  }

  data <- as.matrix(data[, markers])

  if (method == "none") {
    return(data)
  } else if (method == "percentile") {
    return(.percentileNorm(data))
  } else if (method == "minmax") {
    return(.minmaxNorm(data))
  } else if (method == "arsinh") {
    return(.arsinhNnorm(data, cofactor))
  }
  return(scale(data))
}


#' Generate a Self Organizing Map
#' 
#' A self organizing map of the marker intensities is generated and the 
#' prototypes are returned.
#' The grid size is determined automatically
#'
#' @param data the marker intensities
#' @param verbose should the progress be printed out
#' @param size The optimal grid size for the Self Organizing Map
#' @return the self organizing map object
#'
#' @examples
#' data("risom_dat")
#' risomMarkers <- c(
#'   "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
#' )
#' generatePrototypes(risom_dat[, risomMarkers])
#'
#' @export
generatePrototypes <- function(data, verbose = FALSE, size = NULL) {
  message("Now Generating the Self Organizing Map Grid")
  if (is.null(size)) {
    size <- computeGridSize(data)

    # set a lower bound on the grid size
    if (size * size < 25) {
      size <- 5
    } else {
      size <- size
    }

    message("Optimal Grid Size is: ", size)
  } else {
    message("You have provided a grid size of:", size * size)
  }

  # genearte the som grid based on the computed grid size
  sg <- somGrid(xDim = size, yDim = size, topo = "hexagonal")

  # generate the initial prototypes using the first two pcs
  initRes <- somInitPca(as.matrix(data), somGrid = sg)
  message("Now Running the Self Organizing Map Model")

  # generate the self organizing map
  somModel <- batchSom(as.matrix(data), sg,
    prototypes = initRes$prototypes,
    verbose = verbose
  )
  return(somModel)
}


#' Cluster prototypes
#'
#' Cluster the prototypes from the Self Organizing Map 
#' Clustering is done using hierarchical clustering with
#' the average linkage function
#'
#' @param somModel the self organizing map
#' @param numClusters the number of clusters to generate
#' @return the cluster labels
#'
#' @examples
#' data("risom_dat")
#' risomMarkers <- c(
#'   "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
#' )
#' prototypes <- generatePrototypes(risom_dat[, risomMarkers])
#' clusters <- clusterPrototypes(prototypes, 23)
#'
#' @importFrom coop tcosine
#' @importFrom analogue fuse
#' @importFrom psych cor2dist
#' @importFrom FCPS HierarchicalClustering
#' @importFrom stats cor
#' @export
clusterPrototypes <- function(somModel, numClusters = NULL) {
  if (is.null(numClusters)) {
    stop("Please provide the number of clusters")
  }
  # get the prototypes
  prototypes <- somModel$prototypes
  message("Now Clustering the Prototypes")

  # compute the similarity matrices
  pear <- cor(t(prototypes), method = "pearson")
  cosi <- tcosine(prototypes)
  spear <- cor(t(prototypes), method = "spearman")
  eucl <- dist(prototypes)

  # peform multiview integration
  finalDist <- as.matrix(fuse(cor2dist(pear), cor2dist(cosi), cor2dist(spear)),
                         eucl)

  # cluster the final
  clusters <- HierarchicalClustering(finalDist,
    ClusterNo = numClusters,
    Type = "AverageL"
  )$Cls
  message("Now Mapping Clusters to the Original Data")

  clusterAssignment <- clusters[somModel$classif]
  clusterAssignment <- paste0("cluster_", clusterAssignment)

  message("The Prototypes have been Clustered and Mapped Successfully")
  return(clusterAssignment)
}


#' A wrapper function to run the FuseSOM algorithm
#' 
#' This function accepts a matrix, dataframe or a SingleCellExperiment object.
#' For matrices and dataframes, it is assumed that markers are the columns and
#' samples rows.
#'
#' @param data a matrix, dataframe, SingleCellExperiment or SpatialExperiment
#'             object.
#' @param assay the assay of interest if SingleCellExperiment object is used
#' @param markers the markers of interest. If this is not provided, all columns
#'                will be used
#' @param numClusters the number of clusters to be generated from the data
#' @param clusterCol the name of the column to store the clusters in
#' @param size the size of the square grid. eg for a 10X10 grid, size = 10
#' @param verbose should the generation of the Self Organising Map be printed
#'
#' @return A list containing the SOM model and the cluster labels if a dataframe
#' or matrix is provided
#' @return A SingleCellExperiment object with labels in coldata, and SOM model
#' in metadata if a SingleCellExperiment or SpatialExperiment object is provided
#'
#' @examples
#' data("risom_dat")
#' risomMarkers <- c(
#'   "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
#' )
#' res <- runFuseSOM(
#'   risom_dat,
#'   markers = risomMarkers, numClusters = 23, size = 8
#' )
#'
#' @author
#'   Elijah WIllie <ewil3501@uni.sydney.edu.au>
#'
#' @importFrom methods is
#' @importFrom SummarizedExperiment colData colData<-
#' @export
#'
runFuseSOM <- function(data, markers = NULL, numClusters = NULL, assay = NULL,
                       clusterCol = "clusters", size = NULL, verbose = FALSE) {
  if (is.null(numClusters)) {
    stop("Please provide the number of clusters")
  }

  flag <- FALSE

  # if we have a dataframe or a matrix
  if (is(data, "data.frame") || is(data, "matrix")) {
    message("You have provided a dataset of class ", class(data)[[1]])
    # if no markers are given, make sure all the columns are numeric
    if (is.null(markers)) {
      numNumeric <- sum(apply(data, 2, function(x) is.numeric(x)))
      if (numNumeric != ncol(data)) {
        stop(
          "If markers of interest are not provided, make sure the data ",
          "contains all numeric columns"
        )
      }
      dataNew <- data
    } else {
      # extract the markers of interest
      dataNew <- data[, markers]
    }
    # if we have a single cell experiment object
    # if it is SpatialExperiment then is also a  SingleCellExperiment
  } else if (is(data, "SingleCellExperiment")) {
    flag <- TRUE
    message("You have provided a dataset of class ", class(data)[[1]])

    # make sure an assay is provided
    if (is.null(assay)) {
      stop(
        "If a ",
        class(data)[[1]],
        " make sure the appropriate assay is provided as well"
      )
    }
    dataNew <- t(assay(data, assay))

    # again if no markers are given, make sure all the columns are numeric
    if (is.null(markers)) {
      numNumeric <- sum(apply(dataNew, 2, function(x) is.numeric(x)))
      if (numNumeric != ncol(dataNew)) {
        stop(
          "If markers of interest are not provided, ",
          "make sure the data contains all numeric columns"
        )
      }
    } else {
      # extract the markers of interest
      dataNew <- dataNew[, markers]
    }


    # check if the prototypes have already been generated
    if (!is.null(metadata(data)$SOM)) {
      message(
        "The prototypes have already been generated. Checking to see ",
        "if the current markes were used to generate the prototypes"
      )
      # check to see if the same markers were used to generate the previous SOM
      oldSom <- metadata(data)$SOM
      oldMarkers <- colnames(oldSom$data)

      # if no markers were provided
      if (is.null(markers)) {
        if (identical(oldMarkers, colnames(dataNew))) {
          message(
            "The same markers were used to generate the prototypes. ",
            "Checking to see if the same data was used"
          )
          dataOld <- oldSom$data
          if(identical(dataNew, dataOld)){
            message(
              "The same markers and data was used to generate the prototypes. ",
              "Will proceed to clustering the prototypes"
            )
            clusters <- clusterPrototypes(oldSom, numClusters = numClusters)
            colData(data)$clusters <- clusters
            # update the cluster name
            names(colData(data))[names(colData(data)) == "clusters"] <- clusterCol
            return(data)
            
          } else{
            message(
              "Same markers but a different data was used to generate th prototypes. ",
              "Will regenerate the SOM and then cluster prototypes"
            )
            
          }
          
        } else {
          message(
            "Different markers were used to generate the prototypes. ",
            "Will regenerate the SOM and then cluster prototypes"
          )
        }

        # markers were provided
      } else {
        if (identical(oldMarkers, markers)) {
          message(
            "The same markers were used to generate the prototypes. ",
            "Checking to see if the same data was used"
          )
          dataOld <- oldSom$data
          if(identical(dataNew, dataOld)){
            message(
              "The same markers and data was used to generate the prototypes. ",
              "Will proceed to clustering the prototypes"
            )
            clusters <- clusterPrototypes(oldSom, numClusters = numClusters)
            colData(data)$clusters <- clusters
            # update the cluster name
            names(colData(data))[names(colData(data)) == "clusters"] <- clusterCol
            return(data)
            
          } else{
            message(
              "Same markers but a different data was used to generate th prototypes. ",
              "Will regenerate the SOM and then cluster prototypes"
            )
            
          }
          
        } else {
          message(
            "Different markers were used to generate the prototypes. ",
            "Will regenerate the SOM and then cluster prototypes"
          )
        }
      }
    }
  } else {
    stop(
      "Please provide a dataset of type SingleCellExperiment, ",
      "SpatialExperiment, data.frame or matrix"
    )
  }

  # now we can run the FuseSOM algorithm
  message("Everything looks good. Now running the FuseSOM algorithm")
  dataNew <- apply(dataNew, 2, function(x) as.numeric(x))
  somModel <- generatePrototypes(dataNew, verbose = verbose, size = size)
  clusters <- clusterPrototypes(somModel, numClusters = numClusters)

  message("The FuseSOM algorithm has completed successfully")

  if (flag) {
    colData(data)$clusters <- clusters
    # update the cluster name
    names(colData(data))[names(colData(data)) == "clusters"] <- clusterCol
    # overwrite the previous SOM if it is there
    metadata(data)$SOM <- NULL
    metadata(data)$SOM <- somModel
    return(data)
  } else {
    return(list(model = somModel, clusters = clusters))
  }
}



#' Estimate number of clusters
#' 
#' A function for estimating the number of clusters using various method
#' Methods available are: Discriminant, Distance
#' (Gap, Silhouette, Slope, Jump, and Within Cluster Distance,) and Instability
#'
#' @param data the SOM object generated by generatePrototypes(), or an object
#'             of class SingleCellExperiment or SpatialExperiment.
#' @param method one of Discriminant, Distance, Stability. By default,
#'               everything is run
#' @param kSeq a sequence of the number of clusters to try. Default is 2:20
#'             clusters
#' @return A list containing the cluster estimations if a dataframe
#' or matrix is provided
#' @return A SingleCellExperiment with a cluster estimate in it's metadata
#' if a SingleCellExperiment or SpatialExperiment object is provided
#'
#' @examples
#' data("risom_dat")
#' risomMarkers <- c(
#'   "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
#' )
#' res <- runFuseSOM(risom_dat, markers = risomMarkers, numClusters = 23)
#' res.est.k <- estimateNumCluster(res$model, kSeq = 2:25)
#'
#' @author
#'   Elijah WIllie <ewil3501@uni.sydney.edu.au>
#' @importFrom coop tcosine
#' @importFrom analogue fuse
#' @importFrom psych cor2dist
#' @importFrom FCPS HierarchicalClustering
#' @importFrom methods is
#' @importFrom stats cor
#' @importFrom S4Vectors metadata metadata<-
#' @export
#'
estimateNumCluster <- function(data,
                               method = c("Discriminant", "Distance"),
                               kSeq = 2:20) {
  if (1 %in% kSeq) {
    stop("Please select a k sequence starting with 2: {2,3,...K}!")
  }

  flag <- FALSE

  # if we have a single cell experiment or spatial experiment object
  if (is(data, "SingleCellExperiment") || is(data, "SpatialExperiment")) {
    flag <- TRUE
    message("You have provided a dataset of class: ", class(data)[[1]])
    somModel <- data@metadata$SOM
  } else { # if we just have the som model
    somModel <- data
  }

  # extract the prototypes from the model
  prototypes <- somModel$prototypes

  kDiscr <- NULL
  if ("Discriminant" %in% method) {
    message("Now Computing the Number of Clusters using Discriminant Analysis")
    # the minimum number of clusters
    nMin <- 10

    # compute the similarity matrices
    pear <- cor(t(prototypes), method = "pearson")
    cosi <- tcosine(prototypes)
    spear <- cor(t(prototypes), method = "spearman")

    # Get the multiview integration
    finalDist <- as.matrix(
      fuse(cor2dist(pear), cor2dist(cosi), cor2dist(spear))
    )

    # estimate the number of clusters uisng discriminant analysis
    kDiscr <- .runDiscriminant(finalDist, nMin)
  }

  # compute the number of clusters using distance methods
  kDist <- NULL
  if ("Distance" %in% method) {
    message("Now Computing The Number Of Clusters Using Distance Analysis")

    # get the distances
    kDist <- .cDistance(prototypes, kSeq = kSeq)

    # compute the optimal k values using the elbow method
    kDist$kGap <- .computeElbow(kDist$Gaps)
    kDist$kSlope <- .computeElbow(kDist$Slopes)
    kDist$kJump <- .computeElbow(kDist$Jumps)
    kDist$kWCD <- .computeElbow(kDist$WCD)
    kDist$kSil <- .computeElbow(kDist$Silhouettes)
  }

  outList <- list(
    "Discriminant" = kDiscr,
    "Distance" = kDist
  )


  if (flag) {
    metadata(data) <- append(metadata(data), list(clusterEstimation = outList))
    return(data)
  } else {
    return(outList)
  }
}



#' Generate elbow plots
#' 
#' A function generating the elbow plot for the optimal number of clusters
#' returned by the estimateNumcluster() function
#' Methods available are:
#' Gap, Silhouette, Slope, Jump, and Within Cluster Distance(WCD)
#'
#' @param data a Self Organizing Map object generated by generatePrototypes(),
#'  or an object of class SingleCellExperiment or SpatialExperiment
#' @param method one of 'jump', 'slope', 'wcd', 'gap', or 'silhouette'
#' @return an elbow plot object where the optimal number of clusters is marked
#'
#' @examples
#' data("risom_dat")
#' risomMarkers <- c(
#'   "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
#' )
#' res <- runFuseSOM(risom_dat, markers = risomMarkers, numClusters = 23)
#' resEstK <- estimateNumCluster(res$model, kSeq = 2:25)
#' p <- optiPlot(resEstK, method = "jump")
#'
#' @author
#'   Elijah WIllie <ewil3501@uni.sydney.edu.au>
#' @import ggplot2
#' @import ggpubr
#' @import stringr
#' @importFrom methods is
#' @export
#'
optiPlot <- function(data, method = "jump") {

  # make sure a valid method is provided
  if (!(method %in% c("jump", "slope", "wcd", "gap", "silhouette"))) {
    stop("Please provide a valid method")
  }

  # if we have a single cell experiment object
  if (is(data, "SingleCellExperiment") || is(data, "SpatialExperiment")) {
    message("You have provided a dataset of class: ", class(data)[[1]])
    kEst <- data@metadata$clusterEstimation
  } else { # if we just have the som model
    kEst <- data
  }

  # extract the relevant information for the method provided
  if (method == "jump") {
    method <- stringr::str_to_title(method)
    jumps <- kEst$Distance$Jumps
    plotData <- data.frame(Clusters = seq_along(jumps), Jump = jumps)
    kOpti <- kEst$Distance$kJump
  } else if (method == "slope") {
    method <- stringr::str_to_title(method)
    slopes <- kEst$Distance$Slopes
    plotData <- data.frame(Clusters = seq_along(slopes), Slope = slopes)
    kOpti <- kEst$Distance$kSlope
  } else if (method == "wcd") {
    method <- stringr::str_to_upper(method)
    wcds <- kEst$Distance$WCD
    plotData <- data.frame(Clusters = seq_along(wcds), WCD = wcds)
    kOpti <- kEst$Distance$kWCD
  } else if (method == "gap") {
    method <- stringr::str_to_title(method)
    gaps <- kEst$Distance$Gaps
    plotData <- data.frame(Clusters = seq_along(gaps), Gap = gaps)
    kOpti <- kEst$Distance$kGap
  } else {
    method <- stringr::str_to_title(method)
    silhouettes <- kEst$Distance$Silhouettes
    plotData <- data.frame(
      Clusters = seq_along(silhouettes),
      Silhouette = silhouettes
    )
    kOpti <- kEst$Distance$kSil
  }

  # plot the data
  pOpti <- ggpubr::ggline(
    plotData,
    x = "Clusters", y = method, group = 1, color = "steelblue"
  ) +
    geom_vline(xintercept = kOpti, linetype = 2, color = "steelblue") +
    labs(
      y = paste(method, "statistic (k)"), x = "Number of clusters k",
      title = "Optimal number of clusters using the elbow method"
    ) +
    theme(
      plot.title = element_text(
        color = "Black", size = 14, face = "bold", hjust = 0.5
      ),
      axis.title.x = element_text(color = "Black", size = 14, face = "bold"),
      axis.title.y = element_text(color = "Black", size = 14, face = "bold")
    )

  return(pOpti)
}



#' Generate expression heatmap
#' 
#' A function for generating a heat map of marker expression across clusters
#'
#' @param data a matrix or dataframe where the rows are samples and columns are
#' markers
#' @param markers a list of markers of interest. If not provided, all columns
#' will be used
#' @param clusters a vector of cluster labels
#' @param threshold the value to threshold the marker expression at
#' @param clusterMarkers should the rows(markers) of the heatmap be clustered
#' @param fontSize the size of the text on the heatmap
#' @return a heatmap with the markers in the rows and clusters in the columns
#'
#' @examples
#' data("risom_dat")
#' risomMarkers <- c(
#'   "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
#' )
#' res <- runFuseSOM(risom_dat, markers = risomMarkers, numClusters = 23)
#' p.heat <- markerHeatmap(risom_dat, risomMarkers, clusters = res$clusters)
#'
#' @author
#'   Elijah WIllie <ewil3501@uni.sydney.edu.au>
#' @importFrom ggplotify as.ggplot
#' @importFrom pheatmap pheatmap
#' @importFrom methods is
#' @importFrom stats aggregate sd
#' @export
#'
markerHeatmap <- function(data, markers = NULL, clusters = NULL, threshold = 2,
                          clusterMarkers = FALSE, fontSize = 14) {

  # do some house keeping
  if (is.null(clusters)) {
    stop("Please provide a vector of cluster labels")
  }

  if (!(is(data, "matrix") || is(data, "data.frame"))) {
    stop("Make sure you are passing in a dataframe or matrix")
  }


  # again if no markers are given, make sure all the columns are numeric
  if (is.null(markers)) {
    numNumeric <- sum(apply(data, 2, function(x) is.numeric(x)))
    if (numNumeric != ncol(data)) {
      stop(
        "If markers of interest are not provided, ",
        "make sure the data contains all numeric columns"
      )
    }
    message("No markers provided, will be using all columns as markers")
    markers <- colnames(data)
  }

  features <- data[, markers]
  # do some wrangling to get it in the proper format
  featuresHeatmap <- aggregate(
    . ~ as.character(clusters),
    features[, markers],
    mean
  )
  rownames(featuresHeatmap) <- featuresHeatmap[, 1]
  featuresHeatmap <- featuresHeatmap[, -1]

  # compute the marker expression
  featuresHeatmap <- sweep(featuresHeatmap, 2, colMeans(featuresHeatmap), "-")
  featuresHeatmap <- sweep(
    featuresHeatmap, 2, apply(featuresHeatmap, 2, sd), "/"
  )
  featuresHeatmap[featuresHeatmap > threshold] <- threshold
  featuresHeatmap[featuresHeatmap < -threshold] <- -threshold

  # compute the heatmap annotations
  annotationRow <- data.frame(Clusters = rownames(featuresHeatmap))

  rn <- rownames(featuresHeatmap)
  featuresHeatmap <- as.matrix(featuresHeatmap)
  rownames(featuresHeatmap) <- rn
  rownames(annotationRow) <- rownames(featuresHeatmap)

  gapRows <- which(!duplicated(substr(rownames(featuresHeatmap), 1, 2)))[-1] - 1

  pHeat <- ggplotify::as.ggplot(pheatmap(featuresHeatmap,
    gaps_row = gapRows,
    annotation_row = annotationRow, annotation_legend = FALSE,
    cluster_cols = clusterMarkers, cluster_rows = FALSE,
    fontsize = fontSize
  ))
  return(pHeat)
}

---
title: "FuseSOM package manual"
author: "Elijah Willie"
date: "`r Sys.Date()`"
output:
    BiocStyle::html_document:
        toc: true
vignette: >
  %\VignetteIndexEntry{FuseSOM package manual}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

```{r knitr-options, echo=FALSE, message=FALSE, warning=FALSE}
library(knitr)
opts_chunk$set(fig.align = 'center', fig.width = 6, fig.height = 5, dev = 'png')
```

# Installation
```{r, eval = FALSE}
if (!require("BiocManager"))
    install.packages("BiocManager")
BiocManager::install("FuseSOM")
```


# Introduction

A correlation based multiview self organizing map for the characterization of cell types (`FuseSOM`) is a tool for unsupervised clustering. `FuseSOM` is robust and achieves high accuracy by combining a `Self Organizing Map` architecture and a `Multiview` integration of correlation based metrics to cluster highly multiplexed in situ imaging cytometry assays. The `FuseSOM` pipeline has been streamlined and accepts currently used data structures including `SingleCellExperiment` and `SpatialExperiment` objects as well as `DataFrames`.

# Disclaimer

This is purely a tool generated for clustering and as such it does not provide any means for QC and feature selection. It is advisable that the user first use other tools for quality control and feature selection before running `FuseSOM`.

# Getting Started

## `FuseSOM` Matrix Input

If you have a matrix containing expression data that was QCed and normalised by some other tool, the next step is to run the `FuseSOM` algorithm.This can be done by calling the `runFuseSOM()` function which takes in the matrix of interest where the columns are markers and the rows are observations, the makers of interest (if this is not provided, it is assumed that all columns are markers), and the number of clusters.

```{r, message=FALSE, warning=FALSE}
# load FuseSOM
library(FuseSOM)

```

Next we will load in the [`Risom et al`](https://www.sciencedirect.com/science/article/pii/S0092867421014860?via%3Dihub) dataset and run it through the FuseSOM pipeline. This dataset profiles the spatial landscape of ductal carcinoma in situ (DCIS), which is a pre-invasive lesion that is thought to be a precursor to invasive breast cancer (IBC). The key conclusion of this manuscript (amongst others) is that spatial information about cells can be used to predict disease progression in patients.We will also be using the markers used in the original study. 

```{r}
# load in the data
data("risom_dat")

# define the markers of interest
risomMarkers <- c('CD45','SMA','CK7','CK5','VIM','CD31','PanKRT','ECAD',
                   'Tryptase','MPO','CD20','CD3','CD8','CD4','CD14','CD68','FAP',
                   'CD36','CD11c','HLADRDPDQ','P63','CD44')

# we will be using the manual_gating_phenotype as the true cell type to gauge 
# performance
names(risom_dat)[names(risom_dat) == 'manual_gating_phenotype'] <- 'CellType'

```

Now that we have loaded the data and define the markers of interest. We can run the `FuseSOM` algorithm. We have provided a function `runFuseSOM` that runs the pipeline from top to bottom and returns the cluster labels as well as the `Self Organizing Map` model.
```{r}
risomRes <- runFuseSOM(data = risom_dat, markers = risomMarkers, 
                        numClusters = 23)
```


Lets look at the distribution of the clusters.
```{r}
# get the distribution of the clusters
table(risomRes$clusters)/sum(table(risomRes$clusters))

```

Looks like `cluster_1` has about $32\%$ of the cells which is interesting.
Next, lets generate a heatmap of the marker expression for each cluster.

```{r}
risomHeat <- FuseSOM::markerHeatmap(data = risom_dat, markers = risomMarkers,
                            clusters = risomRes$clusters, clusterMarkers = TRUE)
```

## Using `FuseSOM` to estimate the number of clusters
`FuseSOM` also provides functionality for estimating the number of clusters in a dataset using three classes of methods including:

1.  Discriminant based method.
    + A method developed in house based on discriminant based maximum clusterability projection pursuit
2.  Distance based methods which includes:
    + The Gap Statistic
    + The Jump Statistic
    + The Slope Statistic
    + The Within Cluster Dissimilarity Statistic
    + The Silhouette Statistic

We can estimate the number of clusters using the `estimateNumCluster`. Run `help(estimateNumCluster)` to see it's complete functionality.

```{r}
# lets estimate the number of clusters using all the methods
# original clustering has 23 clusters so we will set kseq from 2:25
# we pass it the som model generated in the previous step
risomKest <- estimateNumCluster(data = risomRes$model, kSeq = 2:25, 
                                  method = c("Discriminant", "Distance"))

```
We can then use this result to determine the best number of clusters for this dataset based on the different metrics. The `FuseSOM` package provides a plotting function (`optiPlot`) which generates an elbow plot with the optimal value for the number of clusters for the distance based methods. See below

```{r}
# what is the best number of clusters determined by the discriminant method?
# optimal number of clusters according to the discriminant method is 7
risomKest$Discriminant 

# we can plot the results using the optiplot function
pSlope <- optiPlot(risomKest, method = 'slope')
pSlope
pJump <- optiPlot(risomKest, method = 'jump')
pJump
pWcd <- optiPlot(risomKest, method = 'wcd')
pWcd
pGap <- optiPlot(risomKest, method = 'gap')
pGap
pSil <- optiPlot(risomKest, method = 'silhouette')
pSil

```
From the plots, we see that the `Jump` statistics almost perfectly capture the number of clusters. The `Gap` method is a close second with $15$ clusters. All the other methods significantly underestimates the number of clusters.

## `FuseSOM` Sinlge Cell Epxeriment object as input.

The `FuseSOM` algorithm is also equipped to take in a `SingleCellExperiment` object as input. The results of the pipeline will be written to either the metada or the colData fields. See below.

First we create a `SingleCellExperiment` object
```{r, message=FALSE, warning=FALSE}
library(SingleCellExperiment)

# create a singelcellexperiment object
colDat <- risom_dat[, setdiff(colnames(risom_dat), risomMarkers)]
sce <- SingleCellExperiment(assays = list(counts = t(risom_dat)),
                                 colData = colDat)

sce
```

Next we pass it to the `runFuseSOM()` function. Here, we can provide the assay in which the data is stored and what name to store the clusters under in the colData section. Note that the `Self Organizing Map` that is generated will be stored in the metadata field.

```{r}
risomRessce <- runFuseSOM(sce, markers = risomMarkers, assay = 'counts', 
                      numClusters = 23, verbose = FALSE)

colnames(colData(risomRessce))
names(metadata(risomRessce))
```
Notice how the there is now a clusters column in the colData and SOM field in the metadata. You can run this function again with a new set of cluster number. If you provide a new name for the clusters, it will be stored under that new column, else, it will overwrite the current clusters column. Running it again on the same object will overwrite the SOM field in the metadata.

Just like before, lets plot the heatmap of the resulting clusters across all markers.
```{r}
data <- risom_dat[, risomMarkers] # get the original data used
clusters <- colData(risomRessce)$clusters # extract the clusters from the sce object
# generate the heatmap
risomHeatsce <- markerHeatmap(data = risom_dat, markers = risomMarkers,
                            clusters = clusters, clusterMarkers = TRUE)
```

## Using `FuseSOM` to estimate the number of clusters for single cell experiment objects
Just like before, we can estimate the number of clusters 
```{r}
# lets estimate the number of clusters using all the methods
# original clustering has 23 clusters so we will set kseq from 2:25
# now we pass it a singlecellexperiment object instead of the som model as before
# this will return a singelcellexperiment object where the metatdata contains the
# cluster estimation information
risomRessce <- estimateNumCluster(data = risomRessce, kSeq = 2:25, 
                                  method = c("Discriminant", "Distance"))

names(metadata(risomRessce))
```
Notice how the metadata now contains a `clusterEstimation` field which holds the results from the `estimateNumCluster()` function

We can assess the results in a similar fashion as before
```{r}
# what is the best number of clusters determined by the discriminant method?
# optimal number of clusters according to the discriminant method is 8
metadata(risomRessce)$clusterEstimation$Discriminant 

# we can plot the results using the optiplot function
pSlope <- optiPlot(risomRessce, method = 'slope')
pSlope
pJump <- optiPlot(risomRessce, method = 'jump')
pJump
pWcd <- optiPlot(risomRessce, method = 'wcd')
pWcd
pGap <- optiPlot(risomRessce, method = 'gap')
pGap
pSil <- optiPlot(risomRessce, method = 'silhouette')
pSil

```
Again, we see that the `Jump` statistics almost perfectly capture the number of clusters. The `Gap` method is a close second with $15$ clusters. All the other methods significantly underestimates the number of clusters.

## `FuseSOM` Spatial Epxeriment object as input.
The methodology for `Spatial Epxeriment` is exactly the same as that of `Single Cell Epxeriment`

# sessionInfo()

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilityFunctions.R
\name{.computeElbow}
\alias{.computeElbow}
\title{A function to compute the elbow point given a set of points}
\usage{
.computeElbow(vals)
}
\arguments{
\item{vals}{Values to compute the elbow point of.}
}
\value{
A integer indicating the elbow point of \code{vals}.
}
\description{
A function to compute the elbow point given a set of points
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FuseSOM.R
\docType{package}
\name{FuseSOM}
\alias{FuseSOM}
\title{FuseSOM}
\description{
FuseSOM provides a pipeline for the clustering of highly
multiplexed in situ imaging cytometry assays. This pipeline uses
the Self Organizing Map architecture coupled with Multiview hierarchical
clustering. We also provide functions for normalisation and estimation
of the number of clusters.
}
\details{
The FuseSOM package provides three categories of important functions:
foo, bar and baz.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoreFunctions.R
\name{markerHeatmap}
\alias{markerHeatmap}
\title{Generate expression heatmap}
\usage{
markerHeatmap(
  data,
  markers = NULL,
  clusters = NULL,
  threshold = 2,
  clusterMarkers = FALSE,
  fontSize = 14
)
}
\arguments{
\item{data}{a matrix or dataframe where the rows are samples and columns are
markers}

\item{markers}{a list of markers of interest. If not provided, all columns
will be used}

\item{clusters}{a vector of cluster labels}

\item{threshold}{the value to threshold the marker expression at}

\item{clusterMarkers}{should the rows(markers) of the heatmap be clustered}

\item{fontSize}{the size of the text on the heatmap}
}
\value{
a heatmap with the markers in the rows and clusters in the columns
}
\description{
A function for generating a heat map of marker expression across clusters
}
\examples{
data("risom_dat")
risomMarkers <- c(
  "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
)
res <- runFuseSOM(risom_dat, markers = risomMarkers, numClusters = 23)
p.heat <- markerHeatmap(risom_dat, risomMarkers, clusters = res$clusters)

}
\author{
Elijah WIllie \href{mailto:ewil3501@uni.sydney.edu.au}{ewil3501@uni.sydney.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilityFunctions.R
\name{.minmaxNorm}
\alias{.minmaxNorm}
\title{Function to do min max normalization}
\usage{
.minmaxNorm(x)
}
\arguments{
\item{x}{Matrix to min max nomalize.}
}
\value{
Max normalized version of \code{x}
}
\description{
Function to do min max normalization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runDiscriminant.R
\name{.runDiscriminant}
\alias{.runDiscriminant}
\title{Discriminant cluster estimator}
\usage{
.runDiscriminant(distMat, minClusterSize, alpha = 0.001)
}
\arguments{
\item{distMat}{A distance matrix}

\item{minClusterSize}{The minimum cluster size}

\item{alpha}{a value between 0 and 1 specifying the desired level of cutoff}
}
\value{
Optimal number of clusters
}
\description{
Function to estimate the number of clusters using discriminant analysis
parts of this function is based on the sigclust2 package by Patrick Kimes
see https://github.com/pkimes/sigclust2
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoreFunctions.R
\name{clusterPrototypes}
\alias{clusterPrototypes}
\title{Cluster prototypes}
\usage{
clusterPrototypes(somModel, numClusters = NULL)
}
\arguments{
\item{somModel}{the self organizing map}

\item{numClusters}{the number of clusters to generate}
}
\value{
the cluster labels
}
\description{
Cluster the prototypes from the Self Organizing Map
Clustering is done using hierarchical clustering with
the average linkage function
}
\examples{
data("risom_dat")
risomMarkers <- c(
  "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
)
prototypes <- generatePrototypes(risom_dat[, risomMarkers])
clusters <- clusterPrototypes(prototypes, 23)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilityFunctions.R
\name{.uniformData}
\alias{.uniformData}
\title{Creates uniformly distributed data of same dimensionality as input data
this function was obtained from the Stab package}
\usage{
.uniformData(data)
}
\arguments{
\item{data}{A data matrix.}
}
\value{
Uniform random noise with \code{dim(data)}
}
\description{
Creates uniformly distributed data of same dimensionality as input data
this function was obtained from the Stab package
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{risom_dat}
\alias{risom_dat}
\alias{diabetesData}
\title{IMC Breast Cancer Data
Data from A spatial atlas of breast cancer progression using MIBI-TOF
and tissue transcriptomics}
\format{
An object of class \code{"data.frame"}.
}
\source{
Mendeley Data, \url{https://data.mendeley.com/datasets/d87vg86zd8/3}
}
\usage{
data(risom_dat)
}
\description{
IMC Breast Cancer Data
Data from A spatial atlas of breast cancer progression using MIBI-TOF
and tissue transcriptomics
}
\references{
T. Risom, et al.
Transition to invasive breast cancer is associated with progressive changes
in the structure and composition of tumor stroma Cell, 185 (2022), pp. 299-310
(\href{https://www.sciencedirect.com/science/article/pii/S0092867421014860?via\%3Dihub}{ScienceDirect})
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoreFunctions.R
\name{runFuseSOM}
\alias{runFuseSOM}
\title{A wrapper function to run the FuseSOM algorithm}
\usage{
runFuseSOM(
  data,
  markers = NULL,
  numClusters = NULL,
  assay = NULL,
  clusterCol = "clusters",
  size = NULL,
  verbose = FALSE
)
}
\arguments{
\item{data}{a matrix, dataframe, SingleCellExperiment or SpatialExperiment
object.}

\item{markers}{the markers of interest. If this is not provided, all columns
will be used}

\item{numClusters}{the number of clusters to be generated from the data}

\item{assay}{the assay of interest if SingleCellExperiment object is used}

\item{clusterCol}{the name of the column to store the clusters in}

\item{size}{the size of the square grid. eg for a 10X10 grid, size = 10}

\item{verbose}{should the generation of the Self Organising Map be printed}
}
\value{
A list containing the SOM model and the cluster labels if a dataframe
or matrix is provided

A SingleCellExperiment object with labels in coldata, and SOM model
in metadata if a SingleCellExperiment or SpatialExperiment object is provided
}
\description{
This function accepts a matrix, dataframe or a SingleCellExperiment object.
For matrices and dataframes, it is assumed that markers are the columns and
samples rows.
}
\examples{
data("risom_dat")
risomMarkers <- c(
  "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
)
res <- runFuseSOM(
  risom_dat,
  markers = risomMarkers, numClusters = 23, size = 8
)

}
\author{
Elijah WIllie \href{mailto:ewil3501@uni.sydney.edu.au}{ewil3501@uni.sydney.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoreFunctions.R
\name{generatePrototypes}
\alias{generatePrototypes}
\title{Generate a Self Organizing Map}
\usage{
generatePrototypes(data, verbose = FALSE, size = NULL)
}
\arguments{
\item{data}{the marker intensities}

\item{verbose}{should the progress be printed out}

\item{size}{The optimal grid size for the Self Organizing Map}
}
\value{
the self organizing map object
}
\description{
A self organizing map of the marker intensities is generated and the
prototypes are returned.
The grid size is determined automatically
}
\examples{
data("risom_dat")
risomMarkers <- c(
  "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
)
generatePrototypes(risom_dat[, risomMarkers])

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoreFunctions.R
\name{estimateNumCluster}
\alias{estimateNumCluster}
\title{Estimate number of clusters}
\usage{
estimateNumCluster(data, method = c("Discriminant", "Distance"), kSeq = 2:20)
}
\arguments{
\item{data}{the SOM object generated by generatePrototypes(), or an object
of class SingleCellExperiment or SpatialExperiment.}

\item{method}{one of Discriminant, Distance, Stability. By default,
everything is run}

\item{kSeq}{a sequence of the number of clusters to try. Default is 2:20
clusters}
}
\value{
A list containing the cluster estimations if a dataframe
or matrix is provided

A SingleCellExperiment with a cluster estimate in it's metadata
if a SingleCellExperiment or SpatialExperiment object is provided
}
\description{
A function for estimating the number of clusters using various method
Methods available are: Discriminant, Distance
(Gap, Silhouette, Slope, Jump, and Within Cluster Distance,) and Instability
}
\examples{
data("risom_dat")
risomMarkers <- c(
  "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
)
res <- runFuseSOM(risom_dat, markers = risomMarkers, numClusters = 23)
res.est.k <- estimateNumCluster(res$model, kSeq = 2:25)

}
\author{
Elijah WIllie \href{mailto:ewil3501@uni.sydney.edu.au}{ewil3501@uni.sydney.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoreFunctions.R
\name{normalizeData}
\alias{normalizeData}
\title{Normalize Marker Intensities}
\usage{
normalizeData(data, markers, method = "none", cofactor = 5)
}
\arguments{
\item{data}{the raw intensity scores.}

\item{markers}{the markers of interest.}

\item{method}{the normalizaton method}

\item{cofactor}{the cofactor for arsinh normalization}
}
\value{
normalised matrix.
}
\description{
The matrix of intensities is normalised based on one of four different method
These methods include Percentile, zscore, arsinh and minmax
}
\examples{
data("risom_dat")
risomMarkers <- c(
  "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
)
normaliseData(risom_dat[, risomMarkers])

}
\author{
Elijah WIllie \href{mailto:ewil3501@uni.sydney.edu.au}{ewil3501@uni.sydney.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoreFunctions.R
\name{optiPlot}
\alias{optiPlot}
\title{Generate elbow plots}
\usage{
optiPlot(data, method = "jump")
}
\arguments{
\item{data}{a Self Organizing Map object generated by generatePrototypes(),
or an object of class SingleCellExperiment or SpatialExperiment}

\item{method}{one of 'jump', 'slope', 'wcd', 'gap', or 'silhouette'}
}
\value{
an elbow plot object where the optimal number of clusters is marked
}
\description{
A function generating the elbow plot for the optimal number of clusters
returned by the estimateNumcluster() function
Methods available are:
Gap, Silhouette, Slope, Jump, and Within Cluster Distance(WCD)
}
\examples{
data("risom_dat")
risomMarkers <- c(
  "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
)
res <- runFuseSOM(risom_dat, markers = risomMarkers, numClusters = 23)
resEstK <- estimateNumCluster(res$model, kSeq = 2:25)
p <- optiPlot(resEstK, method = "jump")

}
\author{
Elijah WIllie \href{mailto:ewil3501@uni.sydney.edu.au}{ewil3501@uni.sydney.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilityFunctions.R
\name{.arsinhNnorm}
\alias{.arsinhNnorm}
\title{Function to do arsinh normalization}
\usage{
.arsinhNnorm(x, cofactor = 5)
}
\arguments{
\item{x}{A numeric or complex vector}

\item{cofactor}{Cofactor of the vector. Default is 5.}
}
\value{
Arsinh normalized vector.
}
\description{
Function to do arsinh normalization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/som.R
\name{somInitPca.default}
\alias{somInitPca.default}
\title{these functions were obtained from https://rdrr.io/rforge/yasomi/
with some major modifications}
\usage{
\method{somInitPca}{default}(data, somGrid, weights, with.princomp = FALSE, ...)
}
\arguments{
\item{data}{The data to which the SOM will be fitted, a matrix or data frame
of observations (which should be scaled)}

\item{somGrid}{A \code{somgrid} object}

\item{weights}{Optional weights for the data points}

\item{with.princomp}{Switch specifying whether the princomp should be used
instead of the prcomp for computing the principal
components when no weights are given (see details)}

\item{...}{not used}
}
\value{
A list containing: \code{prototype}, a matrix containing appropriate
initial prototypes, and \code{data.pca} the results of the PCA conducted
on the data
}
\description{
these functions were obtained from https://rdrr.io/rforge/yasomi/
with some major modifications
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CoreFunctions.R
\name{computeGridSize}
\alias{computeGridSize}
\title{Estimate the optimal grid size}
\usage{
computeGridSize(dataset)
}
\arguments{
\item{dataset}{The optimal grid size.}
}
\value{
the optimal grid size.
}
\description{
The function finds the eigenvalues of the sample covariance matrix.
It will then return the number of significant eigenvalues according to
the Tracy-Widom test.
The function is based on the estKW function from the SC3 package
}
\examples{
data("risom_dat")
risomMarkers <- c(
  "CD45", "SMA", "CK7", "CK5", "VIM", "CD31", "PanKRT", "ECAD"
)
computeGridSize(risom_dat[, risomMarkers])

}
\author{
Elijah WIllie \href{mailto:ewil3501@uni.sydney.edu.au}{ewil3501@uni.sydney.edu.au}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utilityFunctions.R
\name{.percentileNorm}
\alias{.percentileNorm}
\title{Function to do percentile normalizaton}
\usage{
.percentileNorm(x)
}
\arguments{
\item{x}{Matrix to percentile normilse.}
}
\value{
percentile normalized version of \code{x}
}
\description{
Function to do percentile normalizaton
}
